{
  "cells": [
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "# **\\[SOLAR_A\\]** 시뮬레이션 (진행중)\n",
        "\n",
        "JiyunLim  \n",
        "2023-08-30"
      ],
      "id": "03b3e243-c1e5-4ec1-bf00-98b13c0f7b7a"
    },
    {
      "cell_type": "code",
      "execution_count": 1,
      "metadata": {},
      "outputs": [],
      "source": [
        "%run eptrgcn/gconvgru"
      ],
      "id": "8172a892-df81-4a33-8dd9-22b3b95842a8"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "# GConvGRU\n",
        "\n",
        "-   Temporal Layer: GRU\n",
        "-   GNN Layer: Chebyshev"
      ],
      "id": "fbbff4fe-6af6-413c-b1c4-835950cf7aab"
    },
    {
      "cell_type": "code",
      "execution_count": 4,
      "metadata": {},
      "outputs": [],
      "source": [
        "class GConv_GRU(torch.nn.Module):\n",
        "    def __init__(self, node_features, filters):\n",
        "        super(GConv_GRU, self).__init__()\n",
        "        self.recurrent = GConvGRU(node_features, filters, 2)\n",
        "        self.linear = torch.nn.Linear(filters, 1)\n",
        "\n",
        "    def forward(self, x, edge_index, edge_weight):\n",
        "        h = self.recurrent(x, edge_index, edge_weight)\n",
        "        h = F.relu(h)\n",
        "        h = self.linear(h)\n",
        "        return h"
      ],
      "id": "75c2e689-5288-4a93-998c-679618fa59d2"
    },
    {
      "cell_type": "code",
      "execution_count": 25,
      "metadata": {},
      "outputs": [],
      "source": [
        "s1 = Simulator()"
      ],
      "id": "8230d25f-0fc4-477f-b01c-60316e5bb3b6"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [
        {
          "output_type": "stream",
          "name": "stdout",
          "text": [
            "epoch=1  loss=1.04801346448271775957642             \n",
            "epoch=2  loss=0.36528553163935973756195     \n",
            "epoch=3  loss=0.21929601328664047496521     "
          ]
        }
      ],
      "source": [
        "s1.simulate(\n",
        "    n_iteration = 10,\n",
        "    lags = [4, 8, 12, `24],\n",
        "    filters = [4, 8, 16, 32],\n",
        "    epoch = [10, 30],\n",
        "    Model = GConv_GRU\n",
        ")"
      ],
      "id": "92470949-addf-4da6-97a1-d0edfdb38a75"
    },
    {
      "cell_type": "code",
      "execution_count": 10,
      "metadata": {},
      "outputs": [],
      "source": [
        "import os\n",
        "fnames = ['./results/gconvgru/'+l for l in os.listdir('./results/gconvgru') if l[0] != '.'][1:]\n",
        "fnames "
      ],
      "id": "fc8cc1ad-851a-476f-8b37-627b3da800a3"
    },
    {
      "cell_type": "code",
      "execution_count": 13,
      "metadata": {},
      "outputs": [],
      "source": [
        "df = pd.concat([pd.read_csv(fname) for fname in fnames]).reset_index(drop=True)"
      ],
      "id": "72bab376-23f0-4d60-afd4-085c95e0cbf7"
    },
    {
      "cell_type": "code",
      "execution_count": 23,
      "metadata": {},
      "outputs": [],
      "source": [
        "def summary_table(df):\n",
        "    df_summary = df.groupby(by=['method', 'epoch', 'lags','nof_filters']).agg({'mse':[np.mean]}).reset_index()\n",
        "    df_classic = df_summary[df_summary['method']=='classic']\n",
        "    df_proposed = df_summary[df_summary['method']=='proposed'].reset_index()[['method','mse']]\n",
        "    df_compare = pd.concat([df_classic, df_proposed], axis=1)\n",
        "    return df_compare"
      ],
      "id": "e3cdb665-a5ef-43a6-8243-2c0161a1a17c"
    },
    {
      "cell_type": "code",
      "execution_count": 24,
      "metadata": {},
      "outputs": [],
      "source": [
        "summary_table(df)"
      ],
      "id": "329003a7-3c77-41fd-8395-474c08df6210"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "<https://miruetoto.github.io/yechan3/posts/3_Researches/PINKOCTO/SOLAR/2023-08-14-EPT-GConvLSTM-setting-Copy1.out.html>\n",
        "\n",
        "# GConvLSTM\n",
        "\n",
        "-   Temporal Layer: LSTM\n",
        "-   GNN Layer: Chebyshev"
      ],
      "id": "91c673d4-d6ae-4b6f-b95d-9cc9de599564"
    },
    {
      "cell_type": "code",
      "execution_count": 380,
      "metadata": {},
      "outputs": [],
      "source": [
        "class GConv_LSTM(torch.nn.Module):\n",
        "    def __init__(self, node_features, filters):\n",
        "        super(GConv_LSTM, self).__init__()\n",
        "        self.recurrent = GConvLSTM(in_channels = node_features, out_channels = filters, K = 2)\n",
        "        self.linear = torch.nn.Linear(filters, 1)\n",
        "\n",
        "    def forward(self, x, edge_index, edge_weight, h, c):\n",
        "        h_0, c_0 = self.recurrent(x, edge_index, edge_weight, h, c)\n",
        "        h = F.relu(h_0)\n",
        "        h = self.linear(h)\n",
        "        return h, h_0, c_0"
      ],
      "id": "6a3100c1-58db-49ba-9728-0749f181cf8d"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "s2 = Simulator()"
      ],
      "id": "09a63131-d446-42b2-b994-b835edaf05dd"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "s2.simulate(\n",
        "    n_iteration = 1,\n",
        "    lags = [2],\n",
        "    filters = [4],\n",
        "    epoch = [5],\n",
        "    Model = GConv_LSTM\n",
        ")"
      ],
      "id": "41b6d4ce-7552-4491-934b-0f92a45dcb74"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "import os\n",
        "fnames = ['./results/'+l for l in os.listdir('./results/') if l[0] != '.'][1]\n",
        "fnames "
      ],
      "id": "19e2e9b0-52e0-43f9-9744-65aa03c1ceee"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "df = pd.concat([pd.read_csv(fname) for fname in fnames]).reset_index(drop=True)\n",
        "df"
      ],
      "id": "1bd1a472-4fd7-4039-8b76-6ec508d9879e"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "# GC-LSTM\n",
        "\n",
        "-   Temporal Layer: LSTM\n",
        "-   GNN Layer: Chebyshev"
      ],
      "id": "56754ff0-5858-41a0-81c7-a89ebb34a379"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "class GC_LSTM(torch.nn.Module):\n",
        "    def __init__(self, node_features, filters):\n",
        "        super(GC_LSTM, self).__init__()\n",
        "        self.recurrent = GCLSTM(node_features, filters, 2)\n",
        "        self.linear = torch.nn.Linear(filters, 1)\n",
        "\n",
        "    def forward(self, x, edge_index, edge_weight, h, c):\n",
        "        h_0, c_0 = self.recurrent(x, edge_index, edge_weight, h, c)\n",
        "        h = F.relu(h_0)\n",
        "        h = self.linear(h)\n",
        "        return h, h_0, c_0"
      ],
      "id": "bff1e6fd-aaac-4396-862d-8b062041f8b9"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "s3 = Simulator()"
      ],
      "id": "a70c230b-9029-47ae-92fc-655e28f509de"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "s3.simulate(\n",
        "    n_iteration = 1,\n",
        "    lags = [2],\n",
        "    filters = [4],\n",
        "    epoch = [5],\n",
        "    Model = GC_LSTM\n",
        ")"
      ],
      "id": "b74e5d95-9a9d-4dc5-89bf-615a7a643aa3"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "import os\n",
        "fnames = ['./results/'+l for l in os.listdir('./results/') if l[0] != '.'][1]\n",
        "fnames "
      ],
      "id": "1c31e7e6-788d-4c61-9d9c-51f74abe86f7"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "df = pd.concat([pd.read_csv(fname) for fname in fnames]).reset_index(drop=True)\n",
        "df"
      ],
      "id": "364a06b8-b79f-47f4-b991-3f61216fd962"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "# LRGCN\n",
        "\n",
        "-   Temporal Layer\n",
        "-   GNN Layer: RGCN"
      ],
      "id": "f64a8b8f-b546-4821-bc7d-32f950522643"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "class LR_GCN(torch.nn.Module):\n",
        "    def __init__(self, node_features, filters):\n",
        "        super(LR_GCN, self).__init__()\n",
        "        self.recurrent = LRGCN(node_features, filters, 1, 1)\n",
        "        self.linear = torch.nn.Linear(filters, 1)\n",
        "\n",
        "    def forward(self, x, edge_index, edge_weight, h_0, c_0):\n",
        "        h_0, c_0 = self.recurrent(x, edge_index, edge_weight, h_0, c_0)\n",
        "        h = F.relu(h_0)\n",
        "        h = self.linear(h)\n",
        "        return h, h_0, c_0"
      ],
      "id": "0040d2bf-95a5-44d3-b944-4dcf5b4b6e95"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "s4 = Simulator()"
      ],
      "id": "cec6b2cc-3759-47dd-82f5-c32f33d82d65"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "s4.simulate(\n",
        "    n_iteration = 1,\n",
        "    lags = [2],\n",
        "    filters = [4],\n",
        "    epoch = [5],\n",
        "    Model = LR_GCN\n",
        ")"
      ],
      "id": "f46b5fb6-2dcc-49e1-b2fe-10cf6dacce0d"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "import os\n",
        "fnames = ['./results/'+l for l in os.listdir('./results/') if l[0] != '.'][1]\n",
        "fnames "
      ],
      "id": "af06574d-12a2-43f2-9c5b-20df07f2b87d"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "df = pd.concat([pd.read_csv(fname) for fname in fnames]).reset_index(drop=True)\n",
        "df"
      ],
      "id": "f2b49ceb-1e49-490d-9f6a-7551a67e78c4"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "# TGCN\n",
        "\n",
        "-   Temporal Layer: GRU\n",
        "-   GNN Layer: GCN"
      ],
      "id": "4bc58921-32dd-48af-a70d-b6fc0b97e47e"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "class T_GCN(torch.nn.Module):\n",
        "    def __init__(self, node_features, filters):\n",
        "        super(T_GCN, self).__init__()\n",
        "        self.recurrent = TGCN(node_features, filters)\n",
        "        self.linear = torch.nn.Linear(filters, 1)\n",
        "\n",
        "    def forward(self, x, edge_index, edge_weight, prev_hidden_state):\n",
        "        h = self.recurrent(x, edge_index, edge_weight, prev_hidden_state)\n",
        "        y = F.relu(h)\n",
        "        y = self.linear(y)\n",
        "        return y, h"
      ],
      "id": "d5552525-a32f-4d45-8e8a-1d65033727c7"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "s5 = Simulator()"
      ],
      "id": "d8a68cd2-d9f4-41b1-b3cc-cd25a82c53c0"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "s5.simulate(\n",
        "    n_iteration = 1,\n",
        "    lags = [2],\n",
        "    filters = [4],\n",
        "    epoch = [5],\n",
        "    Model = T_GCN\n",
        ")"
      ],
      "id": "035e9753-9ae2-4d60-a32c-0e868e3b09de"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "import os\n",
        "fnames = ['./results/'+l for l in os.listdir('./results/') if l[0] != '.'][1]\n",
        "fnames "
      ],
      "id": "b049ce2c-a4ca-47ac-bc62-e38794fc228a"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "df = pd.concat([pd.read_csv(fname) for fname in fnames]).reset_index(drop=True)\n",
        "df"
      ],
      "id": "dc2573a1-d68f-480b-80a7-6286824b82eb"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "# DCRNN\n",
        "\n",
        "-   Temporal Layer: GRU\n",
        "-   GNN Layer: DiffConv"
      ],
      "id": "ab82aa65-985e-4eab-bfee-76bf06bc7b3c"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "class DC_RNN(torch.nn.Module):\n",
        "    def __init__(self, node_features, filters):\n",
        "        super(DC_RNN, self).__init__()\n",
        "        self.recurrent = DCRNN(node_features, filters, 2)\n",
        "        self.linear = torch.nn.Linear(filters, 1)\n",
        "\n",
        "    def forward(self, x, edge_index, edge_weight):\n",
        "        h = self.recurrent(x, edge_index, edge_weight)\n",
        "        h = F.relu(h)\n",
        "        h = self.linear(h)\n",
        "        return h"
      ],
      "id": "25c6dce3-cc49-44aa-8078-78d02424dc6d"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "s6 = Simulator()"
      ],
      "id": "f7f27d5a-583a-4f8a-93af-a13d2959c491"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "s6.simulate(\n",
        "    n_iteration = 1,\n",
        "    lags = [2],\n",
        "    filters = [4],\n",
        "    epoch = [5],\n",
        "    Model = DC_RNN\n",
        ")"
      ],
      "id": "ffa589c9-cee0-4593-9305-4507c2f0374d"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "import os\n",
        "fnames = ['./results/'+l for l in os.listdir('./results/') if l[0] != '.'][1]\n",
        "fnames "
      ],
      "id": "ec8123b7-2e37-4241-b90d-536082a36a4c"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "df = pd.concat([pd.read_csv(fname) for fname in fnames]).reset_index(drop=True)\n",
        "df"
      ],
      "id": "085b45ee-74cc-4e27-b788-b232cd30163d"
    }
  ],
  "nbformat": 4,
  "nbformat_minor": 5,
  "metadata": {
    "kernelspec": {
      "name": "python3",
      "display_name": "Python 3",
      "language": "python"
    },
    "language_info": {
      "name": "python",
      "codemirror_mode": {
        "name": "ipython",
        "version": "3"
      },
      "file_extension": ".py",
      "mimetype": "text/x-python",
      "nbconvert_exporter": "python",
      "pygments_lexer": "ipython3",
      "version": "3.8.16"
    },
    "toc-showtags": false
  }
}